package extensions.reports.itrium

import java.io.File
import java.lang.Boolean
import javax.inject.Inject
import javax.sql.DataSource

import dto.reports.itrium.PassFilter
import extensions.reports.{GenericFormField, GenericJasperReportDefinition}
import models.reports.itrium._
import net.sf.jasperreports.engine.JRParameter
import play.api.Logger
import play.api.libs.json.Json
import play.api.mvc.{Action, AnyContent, Results}
import utils.reports.{Context, ReverseRouting}

import scala.collection.mutable

class ReportDefinition @Inject()(reverseRouting: ReverseRouting) extends GenericJasperReportDefinition {

  private val logger = Logger(getClass)


  override protected def createField(jrParameter: JRParameter): GenericFormField = {
    if (jrParameter.getName.endsWith("ITRIUM_PASS")) {
      new PassFormField(jrParameter, false)
    } else if (jrParameter.getName.endsWith("ITRIUM_PASSES")) {
      new PassFormField(jrParameter, true)
    } else if (jrParameter.getName.endsWith("ITRIUM_ORGANIZATION")) {
      new OrganizationFormField(jrParameter)
    } else if (jrParameter.getName.endsWith("ITRIUM_ORGANIZATION_NOTLINKED")) {
      new OrganizationFormField(jrParameter, notLinked = true)
    } else if (jrParameter.getName.endsWith("ITRIUM_NEYROSS_NODES")) {
      new NeyrossNodeFormField(jrParameter, multiple = true)
    } else if (jrParameter.getName.endsWith("ITRIUM_NEYROSS_NODE")) {
      new NeyrossNodeFormField(jrParameter, multiple = false)
    } else {
      super.createField(jrParameter)
    }
  }

  override def customAction(path: String): Action[AnyContent] = {
    logger.debug(s"path=$path")

    path match {
      case "pass/search" => findPass
      case "organizations" => listOrganizations
      case "departments" => listDepartments
      case "neyrossNodes" => listNeyrossNodes
      case s if s.startsWith("organizations/") =>
        val parts = s.split("/")
        if (parts.length == 3) {
          parts(2) match {
            case "departments" => findDepartments(parts(1).toLong)
          }
        } else {
          Action { Results.NotFound }
        }
      case s if s.startsWith("enumeratedProperties/") =>
        val parts = s.split("/")
        if (parts.length == 3) {
          parts(2) match {
            case "options" => listEnumeratedPropertyOptions(parts(1).toLong)
          }
        } else {
          Action { Results.NotFound }
        }
      case "elements" => listElements
      case _ =>
        super.customAction(path)
    }
  }

  override def renderFormAction: Action[AnyContent] = {
    val routing = reverseRouting.forDataSource(Context.getDataSourceId)
    Action { request =>
      Results.Ok(views.html.reports.itrium.form(getFields, routing))
    }
  }

  private val passComponent = new PassComponent()
  private val organizationComponent = new OrganizationComponent()
  private val neyrossNodeComponent = new NeyrossNodeComponent()
  private val enumeratedPropertyOptionComponent = new EnumeratedPropertyOptionComponent()
  private val elementComponent = new ElementComponent()

  import json.reports.itrium.OrganizationJsonFormat._
  import json.reports.itrium.PassFilterJsonFormat._
  import json.reports.itrium.PassJsonFormat._
  import json.reports.itrium.NeyrossNodeJsonFormat._
  import json.reports.itrium.EnumeratedPropertyOptionJsonFormat._
  import json.reports.itrium.ElementJsonFormat._

  private def findPass: Action[AnyContent] = {
    val dataSource = Context.getDataSource
    Action { request =>
      val filter = Json.fromJson[PassFilter](request.body.asJson.get).get

      implicit val connection = dataSource.getConnection
      try {
        val passes = passComponent.findPasses(filter)
        Results.Ok(Json.toJson(passes))
      } finally {
        connection.close()
      }
    }
  }

  private def listOrganizations: Action[AnyContent] = {
    val dataSource = Context.getDataSource
    Action { request =>
      implicit val connection = dataSource.getConnection
      try {
        val organizations = organizationComponent.listOrganizations()
        Results.Ok(Json.toJson(organizations))
      } finally {
        connection.close()
      }
    }
  }

  private def listDepartments: Action[AnyContent] = {
    val dataSource = Context.getDataSource
    Action { request =>
      implicit val connection = dataSource.getConnection
      try {
        val departments = organizationComponent.listDepartments()
        Results.Ok(Json.toJson(departments))
      } finally {
        connection.close()
      }
    }
  }

  private def findDepartments(organizationId: Long): Action[AnyContent] = {
    val dataSource = Context.getDataSource
    Action { request =>
      implicit val connection = dataSource.getConnection
      try {
        val departments = organizationComponent.findDepartments(organizationId)
        Results.Ok(Json.toJson(departments))
      } finally {
        connection.close()
      }
    }
  }

  private def listNeyrossNodes: Action[AnyContent] = {
    val dataSource = Context.getDataSource
    Action { request =>
      implicit val connection = dataSource.getConnection
      try {
        val neyrossNodes = neyrossNodeComponent.listNeyrossNodes()
        Results.Ok(Json.toJson(neyrossNodes))
      } finally {
        connection.close()
      }
    }
  }

  private def listEnumeratedPropertyOptions(propertyId: Long): Action[AnyContent] = {
    val dataSource = Context.getDataSource
    Action { request =>
      implicit val connection = dataSource.getConnection
      try {
        val options = enumeratedPropertyOptionComponent.findByPropertyId(propertyId)
        Results.Ok(Json.toJson(options))
      } finally {
        connection.close()
      }
    }
  }

  private def listElements: Action[AnyContent] = {
    val dataSource = Context.getDataSource
    Action { request =>
      implicit val connection = dataSource.getConnection

      val elementType = request.getQueryString("elementType").map(_.toLong).getOrElse(0L)
      val useTypeHierarchy = request.getQueryString("useTypeHierarchy").map(_.toBoolean).getOrElse(false)
      try {
        val elements = elementComponent.list(elementType, useTypeHierarchy)
        Results.Ok(Json.toJson(elements))
      } finally {
        connection.close()
      }
    }
  }

  override protected def generateJasperReport(format: String, data: Map[String, AnyRef], dataSource: DataSource, to: File): File = {
    try {
      val extraParameters = mutable.Map[String, AnyRef]()
      if (format == "xls") {
        extraParameters += (JRParameter.IS_IGNORE_PAGINATION -> new Boolean(true))
      }
      super.generateJasperReport(format, data ++ extraParameters, dataSource, to)
    } catch {
      case t: Throwable =>
        logger.error("error:", t)
        throw t
    }
  }
}
